package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.entities.AllowedPolicyValue;
import gov.anl.aps.dm.portal.model.beans.AllowedPolicyValueFacade;
import gov.anl.aps.dm.portal.model.entities.PolicyType;
import java.util.List;

import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import org.apache.log4j.Logger;

@Named("allowedPolicyValueController")
@SessionScoped
public class AllowedPolicyValueController extends CrudEntityController<AllowedPolicyValue, AllowedPolicyValueFacade>
{

    private PolicyType policyType;

    private static final Logger logger = Logger.getLogger(AllowedPolicyValueController.class.getName());

    @EJB
    private AllowedPolicyValueFacade allowedPolicyValueFacade;

    public AllowedPolicyValueController() {
    }

    public PolicyType getPolicyType() {
        return policyType;
    }

    public void setPolicyType(PolicyType policyType) {
        this.policyType = policyType;
        clear();
    }

    @Override
    protected AllowedPolicyValueFacade getFacade() {
        return allowedPolicyValueFacade;
    }

    @Override
    protected AllowedPolicyValue createEntityInstance() {
        return new AllowedPolicyValue();
    }

    @Override
    public String getEntityTypeName() {
        return "allowedPolicyValue";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public String prepareEdit(AllowedPolicyValue allowedPolicyValue) {
        super.resetListDataModel();
        super.prepareEdit(allowedPolicyValue);
        return "editAllowedPolicyValue?faces-redirect=true";

    }

    @Override
    public void prepareEntityInsert(AllowedPolicyValue allowedPolicyValue) throws ObjectAlreadyExists, MissingProperty {
        if ((allowedPolicyValue.getName() == null) || (allowedPolicyValue.getName().length() == 0)) {
            throw new MissingProperty("Name is missing.");
        }
        AllowedPolicyValue existingAllowedPolicyValue = allowedPolicyValueFacade.findByName(allowedPolicyValue.getName());
        if (existingAllowedPolicyValue != null) {
            throw new ObjectAlreadyExists("Experiment type " + allowedPolicyValue.getName() + " already exists.");
        }
        logger.debug("Inserting new experiment type " + allowedPolicyValue.getName());
    }

    @Override
    public void prepareEntityUpdate(AllowedPolicyValue allowedPolicyValue) throws DmPortalException {
        if ((allowedPolicyValue.getName() == null) || (allowedPolicyValue.getName().length() == 0)) {
            throw new MissingProperty("Name is missing.");
        }
    }
    
      @Override
     protected String getObjectAlreadyExistMessage(AllowedPolicyValue allowedPolicyValue)  {
        if (allowedPolicyValue == null) {
            return null;
        }
        return "Experiment Policy " + allowedPolicyValue.getName() + " already exists."; 
    }

     @Override
    public DataModel createListDataModel() {
        return new ListDataModel((List) allowedPolicyValueFacade.findByPolicyType(policyType.getId()));
    }

    @Override
    public List<AllowedPolicyValue> getAvailableItems() {
        return (List<AllowedPolicyValue>) allowedPolicyValueFacade.findByPolicyType(policyType.getId());
    }
    
    @Override
     public String prepareView(AllowedPolicyValue entity) {
        clear();
        current = entity;
        return "viewAllowedPolicyValue?faces-redirect=true";
    }
     
    @Override
    public String prepareCreate() {
        current = createEntityInstance();
        return "createAllowedPolicyValue?faces-redirect=true";
    }
     @Override
     public void clear() {
         super.resetListDataModel();
     }
   
    @FacesConverter(forClass = AllowedPolicyValue.class)
    public static class AllowedPolicyValueControllerConverter implements Converter
    {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            AllowedPolicyValueController controller = (AllowedPolicyValueController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "allowedPolicyValueController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof AllowedPolicyValue) {
                AllowedPolicyValue o = (AllowedPolicyValue) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + AllowedPolicyValue.class.getName());
            }
        }
    }

}