
package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.constants.RoleTypeName;
import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.exceptions.InvalidDate;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.model.beans.AllowedPolicyValueFacade;
import gov.anl.aps.dm.portal.model.entities.Experiment;
import gov.anl.aps.dm.portal.model.beans.ExperimentFacade;
import gov.anl.aps.dm.portal.model.beans.PolicyTypeFacade;
import gov.anl.aps.dm.portal.model.beans.RoleTypeFacade;
import gov.anl.aps.dm.portal.model.beans.UserExperimentRoleFacade;
import gov.anl.aps.dm.portal.model.beans.UserInfoFacade;
import gov.anl.aps.dm.portal.model.entities.CloneableEntity;
import gov.anl.aps.dm.portal.model.entities.ExperimentPolicy;
import gov.anl.aps.dm.portal.model.entities.PolicyType;
import gov.anl.aps.dm.portal.model.entities.RoleType;
import gov.anl.aps.dm.portal.model.entities.UserExperimentRole;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.utilities.SessionUtility;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import org.apache.log4j.Logger;

@Named("experimentController")
@SessionScoped
public class ExperimentController extends CrudEntityController<Experiment, ExperimentFacade>
{

    private static final Logger logger = Logger.getLogger(ExperimentController.class.getName());

    @EJB
    private ExperimentFacade experimentFacade;

    @EJB
    private UserInfoFacade userInfoFacade;

    @EJB
    private RoleTypeFacade roleTypeFacade;
        
    @EJB
    private UserExperimentRoleFacade userExperimentRoleFacade;
        
    public class ExperimentUser extends CloneableEntity {

        String name;
        int experimentId;
        private final UserInfo user;
        private final boolean[] userRoles;

        public ExperimentUser(int experimentId, UserInfo user) {
            userRoles = new boolean[maxExperimentRoleTypeId + 1];
            this.experimentId = experimentId;
            this.user = user;
        }
        
        public String getBadge() {
            return user.getBadge();
        }
        
        public UserInfo getUser() {
            return user;
        }
        
                
        public String getGlobusUsername() {
            return user.getGlobusUsername();
        }

        public String getUsername() {
            return user.getUsername();
        }

         public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public boolean isIsManager() {
            return userRoles[experimentRoles.get(RoleTypeName.MANAGER).getId()];
        }

        public void setIsManager(boolean isManager) {
            userRoles[experimentRoles.get(RoleTypeName.MANAGER).getId()] = isManager;
        }

        public boolean isIsPI() {
            return userRoles[experimentRoles.get(RoleTypeName.PI).getId()];
        }

        public void setIsPI(boolean isPI) {
            userRoles[experimentRoles.get(RoleTypeName.PI).getId()] = isPI;
        }

        public boolean isIsUser() {
            return userRoles[experimentRoles.get(RoleTypeName.USER).getId()];
        }

        public void setIsUser(boolean isUser) {
            userRoles[experimentRoles.get(RoleTypeName.USER).getId()] = isUser;
        }

        public void setIsInRole(RoleType role, boolean isInRole) {
            userRoles[role.getId()] = isInRole;
        }

        public boolean[] getRoles() {
            return userRoles;
        }

        @Override
        public int hashCode() {
            return user.getId() + experimentId * 100;
        }

        @Override
        public boolean equals(Object object) {
            // TODO: Warning - this method won't work in the case the id fields are not set
            if (!(object instanceof ExperimentUser)) {
                return false;
            }
            return ((user.getId() == ((ExperimentUser) object).user.getId()) && (experimentId == ((ExperimentUser) object).experimentId));
        }
    }

    @SessionScoped
    class ExperimentUsersTable extends DataTableController<ExperimentUser> {

        @Override
        public String getClassName() {
            return "ExperimentUser";
        }

        @Override
        public List<ExperimentUser> findAll() {
            List<UserInfo> list = userInfoFacade.findUsersInExperiment(getCurrent().getId());
            return convertExperimentUsers(list);
        }
        
        List<ExperimentUser> convertExperimentUsers(List<UserInfo> list) {
            if (!initialized) {
                initializeTables();
            }
            logger.debug("converting ExperimentUser ");
            ExperimentUser experimentUser;
            for (UserInfo user : list) {
                if ((experimentUser = experimentUsers.get(user)) == null) {
                    experimentUser = new ExperimentUser(getCurrent().getId(), user);
                    experimentUser.setName(user.getLastName() + ", " + user.getFirstName());
                    experimentUsers.put(user, experimentUser);
                }
                for (RoleType roleType : experimentRoles.values()) {
                    boolean inRole = (user.getExperimetRole(roleType.getId(), getCurrent().getId()) != null);
                    experimentUser.setIsInRole(roleType, inRole);
                }
            }
            return new ArrayList<>(experimentUsers.values());
        }
        
        @Override
        public String getTableName() {
            return "experimentUsersTable";
        }
    }

    @SessionScoped
    class NoExperimentUsersTypeTable extends DataTableController<UserInfo> {

        @Override
        public String getClassName() {
            return "UserInfo";
        }

        @Override
        public List<UserInfo> findAll() {
            resetList();
            return userInfoFacade.findNoUsersInExperiment(getCurrent().getId());
        }
        
        @Override
        public String getTableName() {
            return "noExperimentUsersTable";
        }
    }
    
    private int rows = 25;

    private ExperimentController.ExperimentUsersTable experimentUsersListTable = new ExperimentController.ExperimentUsersTable();
    private ExperimentController.ExperimentUsersTable experimentUsersEditTable = new ExperimentController.ExperimentUsersTable();
    private ExperimentController.NoExperimentUsersTypeTable noExperimentUsersTypeTable = new ExperimentController.NoExperimentUsersTypeTable();

    final private Map<UserInfo, ExperimentUser> experimentUsers = new HashMap<>();
    final private Map<String, RoleType> experimentRoles = new HashMap<>();
    int maxExperimentRoleTypeId = 0;
    boolean initialized = false;

    public ExperimentController() {
        this.experimentUsersListTable = new ExperimentController.ExperimentUsersTable();
        this.experimentUsersEditTable = new ExperimentController.ExperimentUsersTable();
        this.noExperimentUsersTypeTable = new ExperimentController.NoExperimentUsersTypeTable();
    }

    public int getRows() {
        return rows;
    }
    
    @Override
    protected ExperimentFacade getFacade() {
        return experimentFacade;
    }

    @Override
    protected Experiment createEntityInstance() {
        Experiment newExperiment = new Experiment();
        if (!initialized) {
            initializeTables();
        }
        return newExperiment;
    }

    @Override
    public String getEntityTypeName() {
        return "experiment";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public List<Experiment> getAvailableItems() {
        return super.getAvailableItems();
    }

    @Override
    public String prepareEdit(Experiment experiment) {
        clear();
        if (!initialized) {
            initializeTables();
        }
        return super.prepareEdit(experiment);
    }

    private void initializeTables() {
        List<RoleType> roleTypesList = roleTypeFacade.findAll();
        for (RoleType roleType : roleTypesList) {
            if (!roleType.isIsSystemRole()) {
                experimentRoles.put(roleType.getName(), roleType);
                if (roleType.getId() > maxExperimentRoleTypeId) {
                    maxExperimentRoleTypeId = roleType.getId();
                }
            }
        }
        initialized = true;
    }

    @Override
    public void prepareEntityInsert(Experiment experiment) throws ObjectAlreadyExists, InvalidDate, MissingProperty {
        if ((experiment.getName() == null) || (experiment.getName().length() == 0)){
            throw new MissingProperty("Experiment name is missing.");
        }
        Experiment existingExperiment = experimentFacade.findByName(experiment.getName());
        if (existingExperiment != null) {
            throw new ObjectAlreadyExists("Experiment " + experiment.getName() + " already exists.");
        }

        verifyExperiment(experiment);
        logger.debug("Inserting new experiment " + experiment.getName());
    }

    @Override
    public void prepareEntityUpdate(Experiment experiment) throws DmPortalException {
        if ((experiment.getName() == null) || (experiment.getName().length() == 0)) {
            throw new MissingProperty("Experiment name is missing.");
        }
        verifyExperiment(experiment);        
        logger.debug("Updating experiment " + experiment.getName());                
    }

     @Override
     protected String getObjectAlreadyExistMessage(Experiment experiment)  {
        if (experiment == null) {
            return null;
        }
        return "Experiment " + experiment.getName() + " already exists."; 
    }
     
    private void verifyExperiment(Experiment experiment) throws InvalidDate, MissingProperty {       
        if (experiment.getExperimentType() == null) {
            throw new MissingProperty("Experiment Type is missing.");
        }
        
        if ((experiment.getStartDate() != null) && (experiment.getEndDate() != null) && (experiment.getEndDate().before(experiment.getStartDate()))) {
            throw new InvalidDate("Experiment end date is before start date.");
        }
    }
   
// the methods for Experiment User

    @Override
    public void clear() {
        experimentUsersListTable.resetList();
        experimentUsersEditTable.resetList();
        experimentUsers.clear();
        noExperimentUsersTypeTable.resetList();
    }
    
    public ExperimentController.ExperimentUsersTable getExperimentUsersListTable() {
        return experimentUsersListTable;
    }

    public void setExperimentUsersListTable(ExperimentController.ExperimentUsersTable experimentUsersTable) {
        this.experimentUsersListTable = experimentUsersTable;
    }
    
    public ExperimentController.ExperimentUsersTable getExperimentUsersEditTable() {
        return experimentUsersEditTable;
    }

    public void setExperimentUsersEditTable(ExperimentController.ExperimentUsersTable experimentUsersTable) {
        this.experimentUsersEditTable = experimentUsersTable;
    }
    
    public ExperimentController.NoExperimentUsersTypeTable getNoExperimentUsersTypeTable() {
        return noExperimentUsersTypeTable;
    }

    public void setNoExperimentUsersTypeTable(ExperimentController.NoExperimentUsersTypeTable noExperimentUsersTypeTable) {
        this.noExperimentUsersTypeTable = noExperimentUsersTypeTable;
    }
                 
    public void updateRemovedExperimentRoles() {
        UserExperimentRole userExperimentRole;
        for (UserInfo user : experimentUsers.keySet()) {
            ExperimentUser experimentUser = experimentUsers.get(user);
            for (RoleType roleType : experimentRoles.values()) {
                int roleId = roleType.getId();
                userExperimentRole = user.getExperimetRole(roleId, experimentUser.experimentId);
                if ((userExperimentRole != null ) && (!experimentUser.userRoles[roleId]) ) {
                   userExperimentRoleFacade.remove(userExperimentRole);
                }
            }
        }
    }

    @Override
    public String update() {
        boolean updated;
        for (UserInfo user : experimentUsers.keySet()) {
            updated = false;
            ExperimentUser experimentUser = experimentUsers.get(user);
            for (RoleType roleType : experimentRoles.values()) {
                if (!roleType.isIsSystemRole()) {
                    int roleId = roleType.getId();
                    if (user.getExperimetRole(roleId, experimentUser.experimentId) == null) {
                        if (experimentUser.userRoles[roleId]) {
                            UserExperimentRole userExperimentRole = new UserExperimentRole(user.getId(), experimentUser.experimentId, roleId);
                            user.addUserExperimentRole(userExperimentRole);
                            updated = true;
                            logger.debug("adding userExperimentRole " + user.getId() + " " + experimentUser.experimentId + " " + roleId);
                        }
                    }
                    else if (!experimentUser.userRoles[roleId]) {
                            user.removeExperimentRole(roleId, experimentUser.experimentId);
                            updated = true;
                            logger.debug("adding userExperimentRole " + user.getId() + " " + experimentUser.experimentId + " " + roleId);
                        }
                    }
                    if (updated) {
                        userInfoFacade.edit(user);
                    }
            }
        }
    
        clear();
        return super.update();
    }

   public String getRemovedUserName() {
        if (experimentUsersEditTable.getCurrentObject() != null) {
            return experimentUsersEditTable.getCurrentObject().getUsername();
        }
        return "";
    }
        
    public String removeUser() {
        ExperimentUser experimentUser = experimentUsersEditTable.getCurrentObject();
        UserInfo user = experimentUser.getUser();
        for (RoleType roleType : experimentRoles.values()) {
            int roleId = roleType.getId();
            if (experimentUser.userRoles[roleId]) {
                user.removeExperimentRole(roleId, experimentUser.experimentId);
            }
        }
        userInfoFacade.edit(user);       
        clear();
        return "edit?faces-redirect=true";
    }
    
    public void removeUserRoles() {
        ExperimentUser experimentUser = experimentUsersEditTable.getCurrentObject();
        UserInfo user = experimentUser.getUser();
        List<UserExperimentRole> roleList = user.getUserExperimentRoles(experimentUser.experimentId);
        for (UserExperimentRole userExperimentRole : roleList) {
             userExperimentRoleFacade.remove(userExperimentRole); 
        }
    }
    
    public String addExperimentUser() {
        List<UserInfo> newUsers = noExperimentUsersTypeTable.getSelectedObjectList();
        if (newUsers == null) {
            logger.debug("null selected list");
            return null;
        } else if (newUsers.isEmpty()) {
            logger.debug("empty selected list");
            return null;
        } else {
            try {
                UserExperimentRole userExperimentRole;
                for (UserInfo user : newUsers) {
                    userExperimentRole = new UserExperimentRole(user.getId(), current.getId(), experimentRoles.get(RoleTypeName.USER).getId()); // set the User
                    user.getUserExperimentRoleList().add(userExperimentRole);
                    userInfoFacade.edit(user);
                }
                clear();
                return "edit?faces-redirect=true";
            } catch (RuntimeException ex) {
                SessionUtility.addErrorMessage("Error", "Could not update UserInfo" + ": " + ex.getMessage());
                return null;
            }
        }
    }
    
    public boolean canEditExperiment(Experiment experiment) {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (experiment == null) {
            return false;
        }
        if (!initialized) {
            initializeTables();
        }
        // user that is Manager or PI can edit experiment
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        int piRoleId = experimentRoles.get(RoleTypeName.PI).getId();
        return (logged.getExperimetRole(managerRoleId, experiment.getId()) != null) || (logged.getExperimetRole(piRoleId, experiment.getId()) != null);
    }
    
    public boolean isRestricted() {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (current == null) {
            return true;
        }
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        int piRoleId = experimentRoles.get(RoleTypeName.PI).getId();
        return (logged.getExperimetRole(managerRoleId, current.getId()) == null) && (logged.getExperimetRole(piRoleId, current.getId()) == null);
    }
    
    public boolean canDeleteExperiment(Experiment experiment) {
        UserInfo logged = (UserInfo) SessionUtility.getUser();
        if (logged == null) {
            return false;
        }
        if (experiment == null) {
            return false;
        }
        if (!initialized) {
            initializeTables();
        }
        int managerRoleId = experimentRoles.get(RoleTypeName.MANAGER).getId();
        return logged.getExperimetRole(managerRoleId, experiment.getId()) != null;
    }
    
    public boolean canAddManager() {
       ExperimentUser logged = experimentUsers.get((UserInfo) SessionUtility.getUser());
        if (logged == null) {
            return false;
        }
        return logged.isIsManager();
    }
    
    public boolean canAddPiAndUser() {
       ExperimentUser logged = experimentUsers.get((UserInfo) SessionUtility.getUser());
         if (logged == null) {
            return false;
        }
        return logged.isIsManager() || logged.isIsPI();
    }
    
    @FacesConverter(forClass = Experiment.class)
    public static class ExperimentControllerConverter implements Converter {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentController controller = (ExperimentController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof Experiment) {
                Experiment o = (Experiment) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + Experiment.class.getName());
            }
        }

    }

}